package org.eclipse.ece.messaging.core;

import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;

import org.junit.Test;
import org.osgi.util.pushstream.PushStream;
import org.osgi.util.pushstream.PushStreamProvider;
import org.osgi.util.pushstream.SimplePushEventSource;

public class TestPushstream {

	@Test
	public void testMerge() throws InterruptedException {
		PushStreamProvider psp = new PushStreamProvider();

		final SimplePushEventSource<String> pes1 = psp.buildSimpleEventSource(String.class).build();
		final SimplePushEventSource<String> pes2 = psp.buildSimpleEventSource(String.class).build();
		final SimplePushEventSource<String> pes3 = psp.buildSimpleEventSource(String.class).build();
		AtomicInteger count = new AtomicInteger();
		TimerTask tt1 = new TimerTask() {

			@Override
			public void run() {
				String text = "msg1-" + count.incrementAndGet();
				pes1.publish(text);

			}
		};
		Timer t1 = new Timer();
		Timer t2 = new Timer();
		TimerTask tt2 = new TimerTask() {

			@Override
			public void run() {
				String text = "msg2-" + count.incrementAndGet();
				pes2.publish(text);

			}
		};
		Timer t3 = new Timer();
		TimerTask tt3 = new TimerTask() {
			
			@Override
			public void run() {
				String text = "msg3-" + count.incrementAndGet();
				pes3.publish(text);
				
			}
		};
		t1.schedule(tt1, 10l, 500l);
		
		PushStream<String> ps1 = psp.createStream(pes1);
//		PushStream<String> ps1 = psp.buildStream(pes1).build();
//		ps1.forEach((s)->System.out.println("ps1: " + s));
		PushStream<String> ps2 = psp.createStream(pes2);
//		PushStream<String> ps2 = psp.buildStream(pes2).build();
		
		PushStream<String> ps3 = psp.createStream(pes3);
//		PushStream<String> ps3 = psp.buildStream(pes3).build();
//		ps2.forEach((s)->System.out.println("ps2: " + s));
		PushStream<String> merge = ps1.merge(ps2).merge(ps3);
		merge.forEach((s)->System.out.println("merge: " + s));
		

		CountDownLatch cdl = new CountDownLatch(1);
		cdl.await(5, TimeUnit.SECONDS);
		
		t2.schedule(tt2, 10l, 600l);
		
		cdl = new CountDownLatch(1);
		cdl.await(5, TimeUnit.SECONDS);
		
		t2.schedule(tt3, 10l, 700l);
		
		cdl = new CountDownLatch(1);
		cdl.await(10, TimeUnit.SECONDS);
		t1.cancel();
		t2.cancel();
		t3.cancel();

	}
	
	@Test
	public void testBackPressure() throws Exception {
		PushStreamProvider psp = new PushStreamProvider();
		
		final SimplePushEventSource<String> pes1 = psp.buildSimpleEventSource(String.class).build();
		AtomicInteger count = new AtomicInteger();
		AtomicLong alt = new AtomicLong(System.currentTimeMillis());
		TimerTask tt1 = new TimerTask() {
			
			@Override
			public void run() {
				long current = System.currentTimeMillis();
				long oldTime = alt.getAndSet(current);
				long time = current - oldTime;
				String text = "msg1-" + count.incrementAndGet();
				pes1.publish(text);
				System.out.println("published: '" + text + "' within " + time + "ms");
				
			}
		};
		Timer t1 = new Timer();
		t1.schedule(tt1, 10l, 1000l);
		AtomicLong al = new AtomicLong(System.currentTimeMillis());
		PushStream<String> ps1 = psp.createStream(pes1);
		ps1.adjustBackPressure((s)->1500).forEach((s)->{
			long current = System.currentTimeMillis();
			long oldTime = al.getAndSet(current);
			long time = current - oldTime;
			System.out.println("received: '" + s + "' within " + time + "ms");
		});
		
//		psp.createEventSourceFromStream(ps1).open(new PushEventConsumer<String>() {
//
//			@Override
//			public long accept(PushEvent<? extends String> event) throws Exception {
//				long current = System.currentTimeMillis();
//				long oldTime = al.getAndSet(current);
//				long time = current - oldTime;
//				System.out.println("received: " + event.getData() + " within " + time + "ms");
//				return 1500;
//			}
//		});
		
		CountDownLatch cdl = new CountDownLatch(1);
		cdl.await(10, TimeUnit.SECONDS);
		System.out.println("Stop timer");
		t1.cancel();
		
		System.out.println("Wait ...");
		cdl = new CountDownLatch(1);
		cdl.await(8, TimeUnit.SECONDS);
		
	}
	
	@Test
	public void testCollection() {
		List<String> collection = new ArrayList<>();
		collection.add("{Hallo}");
		collection.add("{Welt, }");
		collection.add("{ich}");
		collection.add("{bin}");
		collection.add("{Mark}");
		String result = "{Test}".replaceAll("\\{", "[").replaceAll("\\}", "]");
		System.out.println("result: " + result);
		result = collection.stream().map((c)->{return c.replaceAll("\\{", "[").replaceAll("\\}", "]");}).collect(Collectors.joining(","));
		System.out.println("result: " + result);
	}

}
